----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:06:53 04/26/2014 
-- Design Name: 
-- Module Name:    configurable_adder - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity configurable_adder is
	generic ( 
		OPERAND_LENGTH : integer	:= 15;
		EXPONENT_LENGTH : integer	:= 5;
		MANTISSA_LENGTH : integer	:= 9
		);
	Port ( a 		: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			b 			: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			clk		:	in std_logic;
			clk_a		:	in std_logic;
			reset		:	in std_logic;
			new_data	:	in std_logic;
			operation	: in std_logic_vector(2 downto 0);
			result 	: out  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			rdy		: out	std_logic);
end configurable_adder;

architecture Behavioral of configurable_adder is

type state_type is (s0,s1,s2,s3,s4,s5,s6,s7,s8);  --type of state machine.
signal current_s,next_s: state_type;  --current and next state declaration.

-- Signals for adder
signal a_adder			:	std_logic_vector(23 downto 0) := (others => '0');
signal b_adder			:	std_logic_vector(23 downto 0) := (others => '0');
signal add_adder		:	std_logic := '0';
signal ce_adder		:	std_logic := '0';
signal result_adder	:	std_logic_vector(24 downto 0) := (others => '0');

signal res_mantissa : std_logic_vector(MANTISSA_LENGTH-1 downto 0) := (others => '0');
signal big_exponent : std_logic_vector(EXPONENT_LENGTH-1 downto 0) := (others => '0');
signal small_exponent : std_logic_vector(EXPONENT_LENGTH-1 downto 0) := (others => '0');
signal big_mantissa : std_logic_vector(MANTISSA_LENGTH-1 downto 0) := (others => '0');
signal small_mantissa : std_logic_vector(MANTISSA_LENGTH-1 downto 0) := (others => '0');
signal normalization		: std_logic := '0';
signal sign					: std_logic	:= '0';
signal sub_neg				: std_logic := '0';

signal clk_node			: std_logic := '0';
signal delay_clk			: std_logic := '0';
signal normalization_factor			: std_logic_vector(1 downto 0);

--signal diff_exponent : std_logic_vector(MANTISSA_LENGTH downto 0) := (others => '0');

component adder
	port(
		a	:	in std_logic_vector(23 downto 0);
		b	:	in std_logic_vector(23 downto 0);
		clk	:	in std_logic;
		add	:	in std_logic;
		ce		:	in std_logic;
		s		:	out std_logic_vector(24 downto 0)
	);
end component;

attribute KEEP : string;
attribute KEEP of clk_node : signal is "TRUE";

component buf
	port(
		i : in std_logic;
		o : out std_logic
	);
end component;

begin

--	clk_delay_buf : buf
--	port map(
--		i => clk_node,
--		o => delay_clk
--	);


local_fraction_adder_sub : adder
	port map(
		a 		=> a_adder,
		b		=>	b_adder,
		clk	=>	clk_a,
		add	=>	add_adder,
		ce		=> ce_adder,
		s		=>	result_adder
	);

process (clk,reset)
begin
	if (reset='1') then
		current_s <= s0;  --default state on reset.
	elsif (rising_edge(clk)) then
		--clk_node <= clk;
		current_s <= next_s;   --state change.
	--else
	--	clk_node <= clk;
	end if;
end process;

--a_adder <= a(MANTISSA_LENGTH-1 downto 0);
--b_adder <= b(MANTISSA_LENGTH-1 downto 0);
--add_adder <= '1';
--ce_adder <= '1';	

process (current_s, a, b, new_data)
begin
	case current_s is
		when s0 =>
			--result <= (others => '0');
			rdy <= '0';
			a_adder <= (others => '0');
			b_adder <= (others => '0');
			res_mantissa <= (others => '0');
			big_exponent <= (others => '0');
			small_exponent <= (others => '0');
			big_mantissa <= (others => '0');
			small_mantissa <= (others => '0');
			normalization <= '0';
			sign	<= '0';
			sub_neg <= '0';
			
			if new_data = '1' then 
				next_s <= s1;
				-- Subtract exponents to find the biggest
				a_adder(EXPONENT_LENGTH-1 downto 0) <= a(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH);
				b_adder(EXPONENT_LENGTH-1 downto 0) <= b(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH);
				ce_adder <= '1';
				add_adder <= '0';
				
			else next_s <= s0;
			end if;
		when s1 =>
			result <= (others => '0');
			rdy <= '0';
			ce_adder <= '0';
			a_adder <= (others => '0');
			b_adder <= (others => '0');
			res_mantissa <= (others => '0');
			add_adder <= '0';
			sign <= '0';
			
			-- Allocate the biggest number to the biggest exponent and mantissa visa verca for further
			-- operations
			-- B bigger then A
			if (result_adder(MANTISSA_LENGTH) = '1') then 
				big_exponent <= b(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH);
				small_exponent <= a(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH);
				big_mantissa <= b(MANTISSA_LENGTH-1 downto 0);
				small_mantissa <= a(MANTISSA_LENGTH-1 downto 0);
				if(operation(0) = '1') then
					sign <= '1';
					sub_neg <= '1';
				else
				end if;
			-- A bigger or equal to B
			else	
				big_exponent <= a(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH);
				small_exponent <= b(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH);
				big_mantissa <= a(MANTISSA_LENGTH-1 downto 0);
				small_mantissa <= b(MANTISSA_LENGTH-1 downto 0);
			end if;
			
			next_s <= s2;
		when s2 =>
			a_adder <= (others => '0');
			b_adder <= (others => '0');
			result <= (others => '0');
			rdy <= '0';
			res_mantissa <= (others => '0');
			
			-- Calculate the difference between the expononents
			a_adder(EXPONENT_LENGTH-1 downto 0) <= big_exponent;
			b_adder(EXPONENT_LENGTH-1 downto 0) <= small_exponent;
			ce_adder <= '1';
			add_adder <= '0';
			
			next_s <= s3;
		when s3 =>
			ce_adder <= '0';
			a_adder <= (others => '0');
			b_adder <= (others => '0');
			res_mantissa <= (others => '0');
			result <= (others => '0');
			rdy <= '0';
			add_adder <= '0';
			normalization <= '0';
			
			-- Shift the smallest mantissa x places to the right if the difference between the exponents are bigger then 0
			-- If the mantissa is shifted, it is denormalized, going from 1.xxxx to 0.xxxx
			if(to_integer(unsigned(result_adder))) > 0 then
				small_mantissa <= std_logic_vector(unsigned(small_mantissa) srl to_integer(unsigned(result_adder)));
				small_mantissa(MANTISSA_LENGTH-to_integer(unsigned(result_adder))) <= '1';
				normalization <= '1';
			else
			end if;
			next_s <= s4;
		when s4 =>
			a_adder <= (others => '0');
			b_adder <= (others => '0');
			res_mantissa <= (others => '0');
			result <= (others => '0');
			rdy <= '0';
			
			-- Check for addition or subtraction
			if (operation(0) = '0') then
				add_adder <= '1';
			elsif (operation(0) = '1') then
				add_adder <= '0';
			else
			end if;
			
--			if (sub_neg = '1') then
--				a_adder(MANTISSA_LENGTH-1 downto 0) <= small_mantissa;
--				b_adder(MANTISSA_LENGTH-1 downto 0) <= big_mantissa;
--			else

			-- Add mantissas
			-- Assume that A is a normalized numbers 
				a_adder(MANTISSA_LENGTH-1 downto 0) <= big_mantissa;
				a_adder(MANTISSA_LENGTH) <= '1';
				b_adder(MANTISSA_LENGTH-1 downto 0) <= small_mantissa;
				if(normalization = '0') then
					b_adder(MANTISSA_LENGTH) <= '1';
				else
					normalization <= '0';
				end if;
			--end if;
			
			ce_adder <= '1';
			
			next_s <= s5;
		when s5 =>
			a_adder <= (others => '0');
			b_adder <= (others => '0');
			result <= (others => '0');
			ce_adder <= '0';
			add_adder <= '0';
			rdy <= '0';
			res_mantissa <= (others => '0'); 
			
			-- Test if the resulting mantissa is normalized
			if(to_integer(unsigned(result_adder(MANTISSA_LENGTH+1 downto MANTISSA_LENGTH))) > 1) then
				-- Needs to be normalized
				normalization <= '0';
				-- Resulting mantissa is shifted 1 to right, normalized
				res_mantissa <= result_adder(MANTISSA_LENGTH downto 1);
				-- Exponent has to be added
				add_adder <= '1';
				ce_adder <= '1';
				a_adder(EXPONENT_LENGTH-1 downto 0) <= big_exponent;
				if(result_adder(MANTISSA_LENGTH+1 downto MANTISSA_LENGTH) = "10") then
					b_adder(1 downto 0) <= "01";
				else
					b_adder(1 downto 0) <= "10";
				end if;
				next_s <= s6;
			elsif(to_integer(unsigned(result_adder(MANTISSA_LENGTH+1 downto MANTISSA_LENGTH))) < 1) then
				normalization <= '1';
				
				-- Find the position of the leading one
				-- Is this the best way to do it????
				for i in MANTISSA_LENGTH-1 downto 0 loop
					if(result_adder(i) = '1') then
					-- Leading one found
						res_mantissa <= std_logic_vector(unsigned(result_adder(MANTISSA_LENGTH-1 downto 0)) sll (MANTISSA_LENGTH-i));
						a_adder(EXPONENT_LENGTH-1 downto 0) <= big_exponent;
						b_adder(4 downto 0) <= std_logic_vector(to_unsigned(MANTISSA_LENGTH-i,5));
						add_adder <= '0';
						ce_adder <= '1';
						exit;
					else
						
					end if;
				end loop;
				--res_mantissa <= result_adder(MANTISSA_LENGTH-1 downto 0) sll 1;
				
				
				-- NEEDS TO HANDLE MORE THEN ONE SHIFT!!!!
				-- Resulting mantissa is shifted 1 to left
				--res_mantissa(MANTISSA_LENGTH-1 downto 1) <= result_adder(MANTISSA_LENGTH-2 downto 0);
				--res_mantissa(0) <= '0';
				-- Exponent has to be subtracted with 1 since mantissa is shifted
				--a_adder(EXPONENT_LENGTH-1 downto 0) <= big_exponent;
				--b_adder(0) <= '1';
				--add_adder <= '0';
				--ce_adder <= '1';
				next_s <= s6;
			else
				-- Mantissa is normalized
				normalization <= '0';
				res_mantissa <= result_adder(MANTISSA_LENGTH-1 downto 0);
				next_s <= s7;
			end if;

			
				
			--res_mantissa <= result_adder(MANTISSA_LENGTH-1 downto 0);
			--if(result_adder(MANTISSA_LENGTH) = '1' and operation(0) = '0') then
				--res_mantissa <= std_logic_vector(unsigned(result_adder(MANTISSA_LENGTH-1 downto 0)) srl 1);
				
				
				--a_adder(EXPONENT_LENGTH-1 downto 0) <= big_exponent;
				--to_integer(unsigned(result_adder(MANTISSA_LENGTH+1))) - 1;
				--b_adder(1 downto 0) <= std_logic_vector(unsigned(to_integer(unsigned(result_adder(MANTISSA_LENGTH+1 downto MANTISSA_LENGTH)))-1));
				--ce_adder <= '1';
				--add_adder <= '0';
				--normalization <= '1';
			--else
				--res_mantissa <= result_adder(MANTISSA_LENGTH-1 downto 0);
			--end if;
			--next_s <= s6;
			
		-- This state is only needed when exponent has to be changed	
		when s6 =>
			a_adder <= (others => '0');
			b_adder <= (others => '0');
			result <= (others => '0');
			ce_adder <= '0';
			add_adder <= '1';
			rdy <= '0';
			
			big_exponent <= result_adder(EXPONENT_LENGTH-1 downto 0);
			next_s <= s7;
			
		when s7 =>
			a_adder <= (others => '0');
			b_adder <= (others => '0');
			result <= (others => '0');
			ce_adder <= '0';
			add_adder <= '0';
			rdy <= '1';
			
			result(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) <= big_exponent;
			result(MANTISSA_LENGTH-1 downto 0) <= res_mantissa;
			result(OPERAND_LENGTH-1) <= sign;
			
			next_s <= s0;
			
--		when s6 =>
--			a_adder <= (others => '0');
--			b_adder <= (others => '0');
--			result <= (others => '0');
--			ce_adder <= '1';
--			add_adder <= '1';
--			rdy <= '0';
--			
--			-- Set new exponent
--			big_exponent <= result_adder(EXPONENT_LENGTH-1 downto 0);
--			-- If normalization factor is grater then 1, shift mantissa 1 right
--			--if(to_integer(unsigned(normalization_factor) > 1) then
--				--res_mantissa <= std_logic_vector(unsigned(result_adder(MANTISSA_LENGTH+1 down
--			--end if
--			--a_adder(1 downto 0) <= normalization_factor;
--			--b_adder(0) <= '1';
--			
--			next_s <= s7;
--		when s7 =>
--			a_adder <= (others => '0');
--			b_adder <= (others => '0');
--			result <= (others => '0');
--			ce_adder <= '0';
--			add_adder <= '0';
--			rdy <= '0';
--			normalization <= '1';
--			if(to_integer(unsigned(result_adder(1 downto 0))) < 2) then
--				res_mantissa <= std_logic_vector(unsigned(res_mantissa) sll 1);
--			else
--				res_mantissa <= std_logic_vector(unsigned(res_mantissa) srl to_integer(unsigned(result_adder(1 downto 0))));
--			end if;
--			
--			a_adder(EXPONENT_LENGTH-1 downto 0) <= big_exponent;
--			b_adder(1 downto 0) <= result_adder(1 downto 0);
--			ce_adder <= '1';
--			add_adder <= '1';
--			
--			next_s <= s8;
--		when s8 =>
--			a_adder <= (others => '0');
--			b_adder <= (others => '0');
--			ce_adder <= '0';
--			add_adder <= '0';
--			if (normalization = '1') then
--				result(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) <= result_adder(EXPONENT_LENGTH-1 downto 0);
--			else
--				result(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) <= big_exponent;
--			end if;
--			result(MANTISSA_LENGTH-1 downto 0) <= res_mantissa;
--			result(OPERAND_LENGTH-1) <= sign;
--			rdy <= '1';
--			next_s <= s0;
		when others =>
			result <= (others => '0');
			rdy <= '0';
			a_adder <= (others => '0');
			b_adder <= (others => '0');
			res_mantissa <= (others => '0');
			big_exponent <= (others => '0');
			small_exponent <= (others => '0');
			big_mantissa <= (others => '0');
			small_mantissa <= (others => '0');
			add_adder <= '0';
			
			next_s <= s0;
	end case;
end process;


end Behavioral;

