----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:20:53 05/19/2014 
-- Design Name: 
-- Module Name:    configurable_divider - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.std_logic_unsigned.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity configurable_divider is
	generic ( 
		OPERAND_LENGTH : integer	:= 15;
		EXPONENT_LENGTH : integer	:= 5;
		MANTISSA_LENGTH : integer	:= 9
		);
	Port ( a 		: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			b 			: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			clk		:	in std_logic;
			clk_a		:	in std_logic;
			reset		:	in std_logic;
			new_data	:	in std_logic;
			operation	: in std_logic_vector(2 downto 0);
			result 	: out  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			rdy		: out	std_logic
			);
end configurable_divider;

architecture Behavioral of configurable_divider is

	type state_type is (s0,s1,s2,s3);  --type of state machine.
	signal current_s,next_s: state_type;  --current and next state declaration.

	signal leading_zeros_a	:	integer := 0;
	signal leading_zeros_b	:	integer := 0;	
	
	signal a_mantissa			:	std_logic_vector(MANTISSA_LENGTH-1 downto 0);
	signal b_mantissa			:	std_logic_vector(MANTISSA_LENGTH-1 downto 0);
	signal res_mantissa		:	std_logic_vector(MANTISSA_LENGTH-1 downto 0);
	signal res_sign			:	std_logic;
	signal res_exponent		:	std_logic_vector(EXPONENT_LENGTH-1 downto 0);
	
	

begin
	

	process (clk,reset)
	begin
	if (reset='1') then
		current_s <= s0;  --default state on reset.
	elsif (rising_edge(clk)) then
		current_s <= next_s;   --state change.
	end if;
	end process;
	
	process (current_s, a, b, new_data)
	begin
		case current_s is
			when s0 =>
				if new_data = '1' then 
					next_s <= s1;
				else next_s <= s0;
				end if;
				
			when s1 =>
				for i in MANTISSA_LENGTH-1 downto 0 loop
					if(a(i) = '1') then
						leading_zeros_a <= MANTISSA_LENGTH-1-i;
						exit;
					else
					end if;
				end loop;
				
				for j in MANTISSA_LENGTH-1 downto 0 loop
					if(b(j) = '1') then
						leading_zeros_b <= MANTISSA_LENGTH-1-j;
						exit;
					else
					end if;
				end loop;
				
				a_mantissa <= a(MANTISSA_LENGTH-1 downto 0);
				b_mantissa <= b(MANTISSA_LENGTH-1 downto 0);
				
				next_s <= s2;
				
			when s2 =>
				a_mantissa <= std_logic_vector(unsigned(a_mantissa) sll leading_zeros_a);
				b_mantissa <= std_logic_vector(unsigned(b_mantissa) sll leading_zeros_b);
				
				res_exponent <= a(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) - b(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) + (2**(EXPONENT_LENGTH-1)-1) - leading_zeros_a + leading_zeros_b;
				next_s <= s3;
			when s3 =>
				--temp <= leading_zeros_a / leading_zeros_b;
				--res_mantissa(MANTISSA_LENGTH-1 downto 0) <= (a_mantissa(MANTISSA_LENGTH-1 downto 0) / b_mantissa(MANTISSA_LENGTH-1 downto 0));
				res_sign <= a(OPERAND_LENGTH-1) xor b(OPERAND_LENGTH-1);
		end case;
	end process;


end Behavioral;

