
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_unsigned.ALL;
use IEEE.NUMERIC_STD.ALL;

entity configurable_multiplier is
	generic ( 
		OPERAND_LENGTH : integer	:= 32;
		EXPONENT_LENGTH : integer	:= 8;
		MANTISSA_LENGTH : integer	:= 23
		);
	Port ( a 		: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			b 			: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			clk		:	in std_logic;
			--clk_a		:	in std_logic;
			reset		:	in std_logic;
			new_data	:	in std_logic;
			--operation	: in std_logic_vector(5 downto 0);
			result 	: out  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			rdy		: out	std_logic);
end configurable_multiplier;

architecture Behavioral of configurable_multiplier is

signal a_mantissa		: std_logic_vector(MANTISSA_LENGTH downto 0) := (others => '0');
signal b_mantissa		: std_logic_vector(MANTISSA_LENGTH downto 0) := (others => '0');
signal res_mantissa : std_logic_vector(MANTISSA_LENGTH*2+1 downto 0) := (others => '0');
signal res_exponent : std_logic_vector(EXPONENT_LENGTH-1 downto 0) := (others => '0');
signal res_sign	  : std_logic := '0';

constant ZERO 			: std_logic_vector(OPERAND_LENGTH-1 downto 0) := (others => '0');
constant INFINITY		: std_logic_vector(EXPONENT_LENGTH-1 downto 0) := (others => '1');
constant EXPONENT_ZERO	:	std_logic_vector(EXPONENT_LENGTH-1 downto 0) := (others => '0');

-- Signals for adder
--signal a_adder			:	std_logic_vector(23 downto 0) := (others => '0');
--signal b_adder			:	std_logic_vector(23 downto 0) := (others => '0');
--signal add_adder		:	std_logic := '0';
--signal ce_adder		:	std_logic := '0';
--signal result_adder	:	std_logic_vector(24 downto 0) := (others => '0');


type state_type is (s0,s1,s2,s3,s4);  --type of state machine.
signal current_s,next_s: state_type;  --current and next state declaration.

component adder
	port(
		a	:	in std_logic_vector(23 downto 0);
		b	:	in std_logic_vector(23 downto 0);
		clk	:	in std_logic;
		add	:	in std_logic;
		ce		:	in std_logic;
		s		:	out std_logic_vector(24 downto 0)
	);
end component;

begin


--local_fraction_adder_sub : adder
--	port map(
--		a 		=> a_adder,
--		b		=>	b_adder,
--		clk	=>	clk_a,
--		add	=>	add_adder,
--		ce		=> ce_adder,
--		s		=>	result_adder
--	);
	
process (clk,reset)
begin
	if (reset='1') then
		current_s <= s0;  --default state on reset.
	elsif (rising_edge(clk)) then
		--clk_node <= clk;
		current_s <= next_s;   --state change.
	--else
	--	clk_node <= clk;
	end if;
end process;

process (current_s, a, b, new_data)
begin
	case current_s is
		when s0 =>
			a_mantissa <= (others => '0');
			b_mantissa <= (others => '0');
			res_mantissa <= (others => '0');
			res_exponent <= (others => '0');
			res_sign <= '0';
			result <= (others => '0');
			res_sign <= '0';
			rdy <= '0';
			
			if new_data = '1' then 
				next_s <= s1;
				a_mantissa(MANTISSA_LENGTH-1 downto 0) <= a(MANTISSA_LENGTH-1 downto 0);
				a_mantissa(MANTISSA_LENGTH) <= '1';
				b_mantissa(MANTISSA_LENGTH-1 downto 0) <= b(MANTISSA_LENGTH-1 downto 0);
				b_mantissa(MANTISSA_LENGTH) <= '1';
				
				--res_mantissa <= a(MANTISSA_LENGTH-1 downto 0) * b(MANTISSA_LENGTH-1 downto 0);
				next_s <= s1;
			else next_s <= s0;
			end if;
		when s1 =>
			res_mantissa <= (others => '0');
			res_exponent <= (others => '0');
			res_sign <= '0';
			result <= (others => '0');
			res_sign <= '0';
			rdy <= '0';

			res_mantissa <= a_mantissa * b_mantissa;
			res_exponent <= a(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) +  b(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) - (2**(EXPONENT_LENGTH-1)-1);
			next_s <= s2;
			-- Check if overflow => INFINITY 
--			if(a(OPERAND_LENGTH-1) = '1' and b(OPERAND_LENGTH-1) = '1') then
--				--res_exponent <= a(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) +  b(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) - (2**(EXPONENT_LENGTH-1)-1);
--				result(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) <= INFINITY;
--				rdy <= '1';
--				next_s <= s0;
--			-- Check if result is 0
--			elsif(a(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) = EXPONENT_ZERO or  b(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) = EXPONENT_ZERO) then 
--				--result(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) <= INFINITY;
--				next_s <= s0;
--				result <= ZERO;
--				rdy <= '1';
--			else
--				result <= ZERO;
--				next_s <= s0;
--				rdy <= '1';
--			end if;
		when s2 =>
			a_mantissa <= (others => '0');
			b_mantissa <= (others => '0');
			--res_exponent <= (others => '0');
			res_sign <= '0';
			result <= (others => '0');
			rdy <= '0';
			res_sign <= a(OPERAND_LENGTH-1) xor b(OPERAND_LENGTH-1);
			if(res_mantissa(MANTISSA_LENGTH*2+1 downto MANTISSA_LENGTH*2) = "01") then
				next_s <= s3;
			elsif(res_mantissa(MANTISSA_LENGTH*2+1 downto MANTISSA_LENGTH*2) = "10") then
				res_mantissa <= std_logic_vector(unsigned(res_mantissa) srl 1);
				res_exponent <= res_exponent + 1;
				next_s <= s3;
			elsif(res_mantissa(MANTISSA_LENGTH*2+1 downto MANTISSA_LENGTH*2) = "11") then
				res_mantissa <= std_logic_vector(unsigned(res_mantissa) srl 1);
				res_exponent <= res_exponent + 2;
				next_s <= s3;
			else
				next_s <= s0;
			end if;
		when s3 =>
			a_mantissa <= (others => '0');
			b_mantissa <= (others => '0');
			--res_mantissa <= res_mantissa;
			res_exponent <= res_exponent;
			res_sign <= res_sign;
			result <= (others => '0');
			--rdy <= '1';
			
			-- Check for rounding
			if(res_mantissa(MANTISSA_LENGTH*2-MANTISSA_LENGTH-1) = '1') then
				res_mantissa(MANTISSA_LENGTH*2-1 downto MANTISSA_LENGTH*2-MANTISSA_LENGTH) <= res_mantissa(MANTISSA_LENGTH*2-1 downto MANTISSA_LENGTH*2-MANTISSA_LENGTH) + '1';
			else
			end if;
			
			next_s <= s4;
			
			
		when s4 =>
			result(OPERAND_LENGTH-1) <= res_sign;
			result(OPERAND_LENGTH-2 downto OPERAND_LENGTH-1-EXPONENT_LENGTH) <= res_exponent(EXPONENT_LENGTH-1 downto 0);
			result(MANTISSA_LENGTH-1 downto 0) <= res_mantissa(MANTISSA_LENGTH*2-1 downto MANTISSA_LENGTH*2-MANTISSA_LENGTH);
			rdy <= '1';
			
			next_s <= s0;
		when others =>
		
	end case;
end process;

end Behavioral;

