----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:06:13 03/25/2014 
-- Design Name: 
-- Module Name:    fpu_adder_conf - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

library work;
--use work.comppack.all;
use work.fpupack.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity fpu_adder_conf is
	port (
		clk			:	in std_logic;
		a				:	in std_logic_vector(FP_WIDTH-1 downto 0);
		b				:	in std_logic_vector(FP_WIDTH-1 downto 0);
		fpu_op		:	in std_logic_vector(5 downto 0);
		result		:	out std_logic_vector(FP_WIDTH-1 downto 0);
		rst			:	in std_logic;
		ready			:	out std_logic
	);
end fpu_adder_conf;

architecture Behavioral of fpu_adder_conf is

	type state_type is (s0, s1, s2, s3, s4, s5);
	signal current_state, next_state : state_type;

	signal exp_diff : integer := 0;
	signal frac_o	 : std_logic_vector(FRAC_WIDTH-1 downto 0) := (others => '0');
	signal frac_b	 : std_logic_vector(FRAC_WIDTH-1 downto 0) := (others => '0');

begin

process(clk,rst)
begin
	if (rst = '1') then
		current_state <= s0;
	elsif (rising_edge(clk)) then
		current_state <= next_state;
	else
	end if;
end process;

process(current_state, a, b)
begin
	case current_state is
		when s0 =>
			result <= (others => '0');
			ready <= '0';
			next_state <= s1;
		when s1 =>
			if(to_integer(unsigned(a(FP_WIDTH-2 downto FRAC_WIDTH))) >= to_integer(unsigned(b(FP_WIDTH-2 downto FRAC_WIDTH)))) then
				exp_diff <= to_integer(unsigned(a(FP_WIDTH-2 downto FRAC_WIDTH))) - to_integer(unsigned(b(FP_WIDTH-2 downto FRAC_WIDTH)));
				--frac_b <= std_logic_vector(unsigned(b(FRAC_WIDTH-1 downto 0)) srl exp_diff);
			else
				exp_diff <= to_integer(unsigned(b)) - to_integer(unsigned(a));
			end if;
			frac_b <= b(FRAC_WIDTH-1 downto 0);
			next_state <= s2;
		when s2 =>
			frac_b(FP_WIDTH-EXP_WIDTH-exp_diff-1) <= '1';
			next_state <= s3;
		when s3 =>
			frac_b <= std_logic_vector(unsigned(frac_b) srl exp_diff-1);
			frac_o <= std_logic_vector(unsigned(a(FRAC_WIDTH-1 downto 0)) + unsigned(b(FRAC_WIDTH-1 downto 0)));
			next_state <= s4;
		when s4 =>
			result(FP_WIDTH-2 downto FP_WIDTH-2-EXP_WIDTH) <= a(FP_WIDTH-2 downto FP_WIDTH-2-EXP_WIDTH);
			result(FP_WIDTH-1) <= a(FP_WIDTH-1);
			result(FRAC_WIDTH-1 downto 0) <= std_logic_vector(unsigned(a(FRAC_WIDTH-1 downto 0)) + unsigned(frac_b));--frac_o;
			ready <= '0';
			next_state <= s5;
		when s5 =>
			ready <= '1';
			next_state <= s0;
	end case;

end process;
	

end Behavioral;

