
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library work;
use work.all;

entity system_conf is
	generic(
		OPERAND_LENGTH 	: integer := 20;
		EXPONENT_LENGTH 	: integer := 8;
		MANTISSA_LENGTH 	: integer := 11;
		
		EXCEPTIONS			: boolean := false
	);
	port(
		a, b				: 	in std_logic_vector(OPERAND_LENGTH-1 downto 0);
		operation		:	in std_logic_vector(2 downto 0);
		operation_nd	:	in	std_logic;
		clk				:	in std_logic;
		reset				:	in std_logic;
		result_conf		: 	out std_logic_vector(OPERAND_LENGTH-1 downto 0);
		rdy				:	out std_logic;
		
		underflow		:	out std_logic;
		overflow			:	out std_logic;
		invalid_op		:	out std_logic;
		divide_by_zero	:	out std_logic
		);
end system_conf;

architecture Behavioral of system_conf is

	signal ready						:	std_logic := '0';
	signal ready_conf_adder_sub	:	std_logic := '0';
	signal ready_conf_mult			:	std_logic := '0';
	
	signal adder_new_data			:	std_logic := '0';
	signal mult_new_data				:	std_logic := '0';
	
	
	signal result_conf_adder_sub	:	std_logic_vector(OPERAND_LENGTH-1 downto 0) := (others => '0');
	signal result_conf_mult			:	std_logic_vector(OPERAND_LENGTH-1 downto 0) := (others => '0');
	
	--signal fpu_operation				:	std_logic_vector(2 downto 0) := "000";
	
	
	-- Used if EXCEPTIONS is true
	signal sig_underflow				:	std_logic := '0';
	signal sig_overflow				:	std_logic := '0';
	signal sig_invalid_op			:	std_logic := '0';
	signal sig_divide_by_zero		:	std_logic := '0';
	
	
	
	component configurable_adder_sub
		generic ( 
			OPERAND_LENGTH : integer;--	:= 32;
			EXPONENT_LENGTH : integer;--	:= 8;
			MANTISSA_LENGTH : integer--	:= 23
		);
		port(
			a 		: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			b 			: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			clk		:	in std_logic;
			reset		:	in std_logic;
			new_data	:	in std_logic;
			operation	:	in std_logic_vector(2 downto 0);
			result 	: out  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			
			rdy		: out	std_logic
		);
	end component;
	
	component configurable_multiplier
		generic ( 
			OPERAND_LENGTH : integer;--	:= 32;
			EXPONENT_LENGTH : integer;--	:= 8;
			MANTISSA_LENGTH : integer--	:= 23
		);
		port(
			a 		: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			b 			: in  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			clk		:	in std_logic;
			reset		:	in std_logic;
			new_data	:	in std_logic;
			result 	: out  STD_LOGIC_VECTOR (OPERAND_LENGTH-1 downto 0);
			rdy		: out	std_logic
		);
	end component;

begin

		ready <= ready_conf_adder_sub or ready_conf_mult;
		rdy <= ready;
		-- ready_conf_adder_sub or 
		
		underflow <= sig_underflow;
		overflow <= sig_overflow;
		invalid_op <= sig_invalid_op;
		divide_by_zero <= sig_divide_by_zero;
		
		-- Control ready signal
		process(clk, reset, ready_conf_adder_sub, ready_conf_mult)
		begin
			if(reset = '1') then
			else
				if(rising_edge(clk)) then
					if(ready_conf_adder_sub = '1') then
						result_conf <= result_conf_adder_sub;
					elsif(ready_conf_mult = '1') then
						result_conf <= result_conf_mult;
					else
					end if;
					
				end if;
			end if;
		end process;
		
		process(clk, reset, operation, operation_nd, ready)
		begin
			if(reset = '1') then
			else
				if(rising_edge(clk)) then
					if(operation_nd = '1') then
						-- Addition and subtraction
						if(operation = "000" or operation ="001") then
							adder_new_data <= '1';
							mult_new_data <= '0';
						-- Multiplication
						elsif(operation = "010") then
							mult_new_data <= '1';
							adder_new_data <= '0';
						-- Division
						elsif(operation = "011") then
						else
							adder_new_data <= '0';
							mult_new_data <= '0';
						end if;
					elsif(ready = '1') then
						
						adder_new_data <= '0';
						mult_new_data <= '0';
					else
						
					end if;
				end if;
			end if;
		end process;
	
		local_configurable_adder_sub : configurable_adder_sub
		generic map(
			OPERAND_LENGTH	=> OPERAND_LENGTH,
			EXPONENT_LENGTH => EXPONENT_LENGTH,
			MANTISSA_LENGTH => MANTISSA_LENGTH
			)
		port map(
			a					=>	a,
			b					=>	b,
			clk				=> clk,
			reset				=>	reset,
			operation		=> operation,
			new_data			=>	adder_new_data,
			result			=> result_conf_adder_sub,
			rdy				=> ready_conf_adder_sub
		);
		
		local_configurable_mult : configurable_multiplier
		generic map(
			OPERAND_LENGTH	=> OPERAND_LENGTH,
			EXPONENT_LENGTH => EXPONENT_LENGTH,
			MANTISSA_LENGTH => MANTISSA_LENGTH
			)
		port map(
			a					=>	a,
			b					=>	b,
			clk				=> clk,
			reset				=>	reset,
			new_data			=>	mult_new_data,
			result			=> result_conf_mult,
			rdy				=> ready_conf_mult
		);




end Behavioral;

