library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
library IEEE_PROPOSED;
use IEEE_PROPOSED.fixed_float_types.all;
use IEEE_PROPOSED.fixed_pkg.all;


entity system_ieee_fixed is
	generic(
		BIT_WIDTH		:	integer := 32;
		INTEGER_WIDTH	:	integer := 10;
		FRACTION_WIDTH	:	integer := 22
	);
	port(
		clk				: in std_logic;
		reset				: in std_logic;
		a					: in std_logic_vector(BIT_WIDTH-1 downto 0);
		b					: in std_logic_vector(BIT_WIDTH-1 downto 0);
		operation		: in std_logic_vector(5 downto 0);
		operation_nd	: in std_logic;
		sum_add_sub_o	: out std_logic_vector(BIT_WIDTH downto 0);
		sum_mult_o		: out std_logic_vector(2*INTEGER_WIDTH-1+2*FRACTION_WIDTH downto 0);
		ready				: out std_logic
	);
end system_ieee_fixed;

architecture Behavioral of system_ieee_fixed is

	signal afp, bfp		: sfixed(INTEGER_WIDTH-1 downto -FRACTION_WIDTH) := (others => '0');
	signal sum_adder_sub	: sfixed(afp'left + 1 downto afp'right) := (others => '0');
	signal sum_mult		: sfixed(afp'left+bfp'left+1 downto afp'right+bfp'right) := (others => '0');
	--signal sum_div			: ufixed(afp'left-bfp'right+1 downto afp'right-bfp'left) := (others => '0');
	signal sum_div			: sfixed(sfixed_high (afp, '/', bfp) downto sfixed_low (afp, '/', bfp));
	
	type state_type is (s0,s1,s2);  --type of state machine.
	signal current_s,next_s: state_type;  --current and next state declaration.

begin

	afp <= to_sfixed(a, afp'left, afp'right);
	bfp <= to_sfixed(b, bfp'left, bfp'right);
	
	process (clk,reset)
	begin
		if (reset='1') then
			current_s <= s0;  --default state on reset.
		elsif (rising_edge(clk)) then
			current_s <= next_s;   --state change.
		else
		end if;
	end process;
	
	process (current_s, operation_nd)
	begin
		case current_s is
			when s0 =>
				ready <= '0';
				if(operation_nd = '1') then
					next_s <= s1;
				else
					next_s <= s0;
				end if;
			when s1 =>
				ready <= '1';
			when s2 =>
		end case;
	end process;
	
	process(clk, reset)
	begin
		if(reset = '1') then
			sum_add_sub_o <= (others => '0');
			sum_mult_o <= (others => '0');
		elsif(rising_edge(clk)) then
			if(operation_nd = '1') then
				if(operation = "000000") then
					sum_adder_sub <= afp + bfp;
				elsif(operation = "000001") then
					sum_adder_sub <= afp - bfp;
				elsif(operation = "000010") then
					sum_mult <= afp * bfp;
				elsif(operation = "000011") then
--					sum_div <= divide(	l => afp,
--								r => bfp,
--								round_style => fixed_round,
--								guard_bits 	=> 3);
					--sum_div <= afp/bfp;
				else
				end if;
			else
			end if;
		else
		end if;
		sum_add_sub_o <= to_slv(sum_adder_sub);
		sum_mult_o <= to_slv(sum_mult);
	end process;


end Behavioral;

