
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
library IEEE_PROPOSED;
use IEEE_PROPOSED.float_pkg.all;
use IEEE_PROPOSED.fixed_float_types.all;

entity system_ieee_float is
	port(
		clk				:	in std_logic;
		reset				: 	in std_logic;
		operation_nd	: 	in std_logic;
		operation		:	in std_logic_vector(5 downto 0);
		a					:	in std_logic_vector(float_exponent_width+float_fraction_width downto 0);
		b					:	in std_logic_vector(float_exponent_width+float_fraction_width downto 0);
		sum				:	out std_logic_vector(float_exponent_width+float_fraction_width downto 0);
		ready				:	out std_logic
	);
end system_ieee_float;

architecture Behavioral of system_ieee_float is

	signal afp, bfp, sumfp : float(float_exponent_width downto - float_fraction_width);
	
	type state_type is (s0,s1,s2);  --type of state machine.
	signal current_s,next_s: state_type;  --current and next state declaration.

begin
	afp <= to_float(a, afp'high, -afp'low);
	bfp <= to_float(b, bfp'high, -bfp'low);
	
	process (clk,reset)
	begin
		if (reset='1') then
			current_s <= s0;  --default state on reset.
		elsif (rising_edge(clk)) then
			current_s <= next_s;   --state change.
		else
		end if;
	end process;
	
	process (current_s, operation_nd)
	begin
		case current_s is
			when s0 =>
				ready <= '0';
				if(operation_nd = '1') then
					next_s <= s1;
				else
					next_s <= s0;
				end if;
			when s1 =>
				ready <= '1';
			when s2 =>
		end case;
	end process;

	process(clk, reset, operation_nd, operation, afp, bfp, sumfp)
	begin
		if(reset = '1') then
			sumfp <= (others => '0');
		elsif(rising_edge(clk)) then
			if(operation_nd = '1') then
				if(operation = "000000") then
					sumfp <= afp + bfp;
				elsif(operation = "000001") then
					sumfp <= afp - bfp;
				elsif(operation = "000010") then
					sumfp <= afp * bfp;
				elsif(operation = "000011") then
					sumfp <= afp / bfp;
				else
				end if;
			end if;
		else
		end if;
		sum <= to_slv(sumfp);
	end process;


end Behavioral;

