`timescale 1ns / 1ps

module system_tb;

	// Inputs
	reg [19:0] a;
	reg [19:0] b;
	reg [2:0] operation;
	reg operation_nd;
	reg clk;
	reg reset;

	// Outputs
	wire [19:0] result_ip;
	wire rdy;
	wire underflow;
	wire overflow;
	wire invalid_op;
	wire divide_by_zero;

	integer   data_file; //file handler
	integer   data_file_result; //file handler
	integer	 scan_file; //file handler	
	reg		[2:0] state;
	integer	counter;
	
	`define NULL 0
	
	parameter zero=0, one=1, two=2, three=3, four=4;

	// Instantiate the Unit Under Test (UUT)
	system uut (
		.clk(clk), 
		.reset(reset),
		.operation_nd(operation_nd),
		.operation(operation),
		.a(a), 
		.b(b), 
		.result_ip(result_ip),
		.rdy(rdy),
		.underflow(underflow),
		.overflow(overflow),
		.invalid_op(invalid_op),
		.divide_by_zero(divide_by_zero)
	);

	initial begin
		// Initialize Inputs
		clk = 0;
		reset = 1;
		a = 0;
		b = 0;
		operation = 0;
		operation_nd = 0;
		counter = 0;

		data_file = $fopen("fpu_custom.dat", "r");
		data_file_result = $fopen("../MATLAB/fpu_testbench_results_ip_64.txt", "w");
		if (data_file == `NULL) begin
			$display("data_file handle is NULL");
			$finish;
		end
		
		if (data_file_result == `NULL) begin
			$display("data_file handle is NULL");
			$finish;
		end
		#200;
		
		reset = 0;
		
		repeat (5000) @ (posedge clk);
		$fclose(data_file);
		$fclose(data_file_result);
		reset = 1;
		#100 
		$finish;

	end
	
	always @(posedge rdy) begin
		#20
		$fwrite(data_file_result, "%h %d\n", result_ip, operation);
	end
	
	always @(posedge state == three) begin
		if(counter == 100)
			reset = 1;
		else
			counter = counter + 1;
	end
	
	always @(state) begin
		case (state)
			zero:
				operation_nd = 0;
			one:
				scan_file = $fscanf(data_file, "%b %b\n", a, b);
			two:
				if(operation == 3)
					operation = 0;
				else
					operation = operation + 1;
			three:
				operation_nd = 1;
			four:
				operation_nd = 0;
		endcase
	end
	
	always @(posedge clk or posedge reset) begin
		if (reset == 1)	
			state = zero;
		else
			case (state)
				zero:
					if(reset == 1)
						state = zero;
					else if(rdy == 0)
						state = one;
					else
						state = zero;
				one:
					state = two;
				two:
					state = three;
				three:
					state = four;
				four:
					//state = zero;
					if(rdy == 0)
						state = four;
					else
						state = zero;
			endcase;
	end
	
	always #5 clk = !clk;
      
endmodule